#![doc = include_str!("../README.md")]
use std::{fs, collections::HashMap, io::BufWriter, sync::Mutex};

mod checksum;
mod error;
mod file;
mod filetype;
pub(crate) mod nix;
mod node;
mod special;
mod stream;

pub use {
    checksum::{Algorithm, Checksum}, error::Error, file::File, filetype::FileType, node::Node, special::Special,
    stream::Stream,
};

pub fn create_archive(path: &str, files: Vec<&str>, algorithm: Algorithm) -> Result<(), Error> {
    let fd = fs::OpenOptions::new()
        .create(true)
        .truncate(true)
        .open(path)?;
    let mut writer = BufWriter::new(fd);
    let mut links = Mutex::new(HashMap::new());
    for f in files.iter() {
        let node = Node::from_path(f, algorithm, &mut links)?;
        node.write(&mut writer)?;
    }
    Ok(())
}
